<?php

require_once(DIR_FS_CATALOG.'ext/modules/payment/verotel_flexpay/FlexPay/Client.php');

class verotel_flexpay_GetOrderStatusException extends Exception {}
class verotel_flexpay extends abstract_payment_module {


    const CONFIG_KEY_BASE = 'MODULE_PAYMENT_VEROTEL_FLEXPAY_';

    const REQUIRES = [
        'email_address',
    ];

    public function __construct() {
        $approve_url = tep_href_link('ext/modules/payment/verotel_flexpay/callback.php', '');
        $success_url = tep_href_link('ext/modules/payment/verotel_flexpay/success.php', '');
        if (function_exists("tep_catalog_href_link")){
            $approve_url = tep_catalog_href_link('ext/modules/payment/verotel_flexpay/callback.php', '');
            $success_url = tep_catalog_href_link('ext/modules/payment/verotel_flexpay/success.php', '');
        }

        parent::__construct();
        $this->public_title = MODULE_PAYMENT_VEROTEL_FLEXPAY_TEXT_PUBLIC_TITLE;
        $this->title = MODULE_PAYMENT_VEROTEL_FLEXPAY_TEXT_TITLE;
        $this->description = sprintf(
            MODULE_PAYMENT_VEROTEL_FLEXPAY_TEXT_DESCRIPTION,
            $approve_url,
            $success_url
        );

        $this->sort_order = defined('MODULE_PAYMENT_VEROTEL_FLEXPAY_SORT_ORDER')
            ? MODULE_PAYMENT_VEROTEL_FLEXPAY_SORT_ORDER : 0;

        $this->cfg_order_status =
            defined('MODULE_PAYMENT_VEROTEL_FLEXPAY_ORDER_STATUS_ID')
                && ((int)MODULE_PAYMENT_VEROTEL_FLEXPAY_ORDER_STATUS_ID) > 0
                    ? (int)MODULE_PAYMENT_VEROTEL_FLEXPAY_ORDER_STATUS_ID : DEFAULT_ORDERS_STATUS_ID;

        $this->order_status = $this->cfg_order_status;

        $this->flexpay_client = NULL;
        if (
            defined('MODULE_PAYMENT_VEROTEL_FLEXPAY_SHOP_ID')
            && defined('MODULE_PAYMENT_VEROTEL_FLEXPAY_SIGNATURE')
            && (int)MODULE_PAYMENT_VEROTEL_FLEXPAY_SHOP_ID > 0
        ) {
            $this->flexpay_client = new Verotel\FlexPay\Client(
                MODULE_PAYMENT_VEROTEL_FLEXPAY_SHOP_ID,
                MODULE_PAYMENT_VEROTEL_FLEXPAY_SIGNATURE,
                Verotel\FlexPay\Brand::create_from_name(MODULE_PAYMENT_VEROTEL_FLEXPAY_BRAND)
            );
        }

        $this->form_action_url = tep_href_link('ext/modules/payment/verotel_flexpay/checkout.php', '');
    }

    protected function get_session_order_id() {
        if (isset($_SESSION['cart_Verotel_FlexPay_pre_order'])) {
            list($order_id, $cart_id, $customer_id) =
                explode('-', $_SESSION['cart_Verotel_FlexPay_pre_order'], 3);

            if ($_SESSION['customer_id'] != $customer_id) {
                unset($_SESSION['cart_Verotel_FlexPay_pre_order']);
                return null;
            }

            return $order_id;
        }
        return null;
    }

    protected function delete_order_if_there_is_no_history() {
        if (isset($_SESSION['cart_Verotel_FlexPay_pre_order'])) {
            $order_id = $this->get_session_order_id();

            if ($order_id != null) {
                $check_query = tep_db_query(
                    'SELECT orders_id FROM orders_status_history WHERE'
                        . ' orders_id = ' . (int)$order_id
                        . ' AND orders_status_id <> ' . $this->get_awaiting_payment_status_id()
                        . ' LIMIT 1'
                );

                if (tep_db_num_rows($check_query) < 1) {
                    tep_delete_order($order_id);
                    unset($_SESSION['cart_Verotel_FlexPay_pre_order']);
                }
            }
        }
    }

    public function selection() {
        $this->delete_order_if_there_is_no_history();

        return parent::selection();
    }

    public function pre_confirmation_check() {
        if (empty($_SESSION['cart']->cartID)) {
            $_SESSION['cartID'] = $_SESSION['cart']->cartID =
                $_SESSION['cart']->generate_cart_id();
        }
    }

    public function confirmation() {
        global $order;

        $this->delete_order_if_there_is_no_history();

        if (!isset($_SESSION['cart_Verotel_FlexPay_pre_order'])) {
            $order->info['order_status'] = $this->get_awaiting_payment_status_id();
            require 'includes/system/segments/checkout/build_order_totals.php';
            require 'includes/system/segments/checkout/insert_order.php';
            $sql_data = [
                'orders_id' => $order->get_id(),
                'orders_status_id' => $order->info['order_status'],
                'date_added' => 'NOW()',
                'customer_notified' => 0,
                'comments' => $order->info['comments'],
            ];
            tep_db_perform('orders_status_history', $sql_data);

            $_SESSION['cart_Verotel_FlexPay_pre_order'] = $order->get_id()
                . '-' . $_SESSION['cartID'] . '-' . $_SESSION['customer_id'];
        }

        return false;
    }

    public function process_button(){
        global $order, $currencies;

        $total = number_format(
            ($order->info['total'] * $currencies->get_value($_SESSION['currency'])),
            $currencies->currencies[$_SESSION['currency']]['decimal_places'],
            '.', ''
        );

        $params = array(
            'priceAmount' =>  $total,
            'priceCurrency' => $_SESSION['currency'],
            'referenceID' => $_SESSION['cart_Verotel_FlexPay_pre_order'],
            'description' => substr(STORE_NAME, 0, 100),
            'email' =>  $order->customer['email_address'],
            'origin' => 'OsCommerceV4',
            'custom1' => $_SESSION['cartID'],
            'custom2' => $_SESSION['customer_id'],
            'custom3' => $order->get_id(),
        );

        return tep_draw_hidden_field('purchaseURL',
            $this->flexpay_client->get_purchase_URL($params));
    }

    public function before_process(){
        if ( !$this->flexpay_client->validate_signature($_GET) ){
            tep_redirect(
                tep_href_link(
                    FILENAME_CHECKOUT_PAYMENT,
                    'payment_error=' . $this->code .
                    '&signature_error=1'
                )
            );
        }

        if ($_SESSION['cart_Verotel_FlexPay_pre_order'] != $_GET['referenceID']) {
            tep_redirect(
                tep_href_link(
                    FILENAME_CHECKOUT_PAYMENT,
                    'payment_error=' . $this->code
                )
            );
        }

        $order_id = $this->get_session_order_id();
        $order = $GLOBALS['order'] = new order($order_id);

        try {
            $this->accept_payment($order, $order_id, $_SESSION['customer_id']);
        } catch (verotel_flexpay_GetOrderStatusException $e) {
            $this->redirect_to_cart();
        }

        tep_redirect(tep_href_link('checkout_success.php'));
        return false;
    }

    public function callback_result(){
        if ( !$this->flexpay_client->validate_signature($_GET) ){
            return 'ERROR - invalid signature';
        }

        $_SESSION['cart_Verotel_FlexPay_pre_order'] = $_GET['referenceID'];
        $_SESSION['customer_id'] = $_GET['custom2'];
        $_SESSION['cartID'] = $_GET['custom1'];

        $customer = $GLOBALS['customer'] = new customer($_SESSION['customer_id']);
        $order_id = $this->get_session_order_id();
        $order = $GLOBALS['order'] = new order($order_id);

        if (!isset($order->info['comments'])){
            $order->info['comments'] = '';
        }

        try {
            $this->accept_payment($order, $order_id, $_SESSION['customer_id']);
        } catch (verotel_flexpay_GetOrderStatusException $e) {
            return 'ERROR - order ' . $order_id . ' not found';
        }

        return 'OK';
    }

    public function get_error(){
        if (isset($_GET['signature_error'])) {
            $error = MODULE_PAYMENT_VEROTEL_FLEXPAY_TEXT_ERROR_SIGNATURE;
        } else {
            $error = MODULE_PAYMENT_VEROTEL_FLEXPAY_TEXT_ERROR_UNKNOWN;
        }

        return array('title' => MODULE_PAYMENT_VEROTEL_FLEXPAY_TEXT_ERROR,
            'error' => $error);
    }

    protected function get_parameters() {
        $this->get_awaiting_payment_status_id();
        $this->get_payment_accepted_status_id();

        return [
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_STATUS' => [
                'title' => 'Enable Verotel FlexPay',
                'value' => 'True',
                'desc' => 'Do you want to accept Verotel FlexPay payments?',
                'set_func' => "tep_cfg_select_option(['True', 'False'], ",
            ],
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_BRAND' => [
                'title' => 'Payment processor',
                'desc' => '',
                'value' => 'Verotel',
                'set_func' => "tep_cfg_pull_down_verotel_flexpay_brand(",
                'use_func' => "tep_cfg_show_verotel_flexpay_brand",
            ],
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_SHOP_ID' => [
                'title' => 'Shop (website) ID',
                'desc' => 'Verotel Shop (website) ID',
            ],
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_SIGNATURE' => [
                'title' => 'Signature Key',
                'desc' => 'You will find it in Verotel Control Center FlexPay Options',
            ],
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_ZONE' => [
                'title' => 'Payment Zone',
                'value' => '0',
                'desc' => 'If a zone is selected, only enable this payment method for that zone.',
                'use_func' => 'tep_get_zone_class_title',
                'set_func' => 'tep_cfg_pull_down_zone_classes(',
            ],
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_SORT_ORDER' => [
                'title' => 'Sort order of display.',
                'value' => '0',
                'desc' => 'Sort order of display. Lowest is displayed first.',
            ],
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_ORDER_STATUS_ID' => [
                'title' => 'Set Order Status',
                'value' => 0,
                'desc' => 'Set the status of orders made with this payment module to this value',
                'set_func' => 'tep_cfg_pull_down_order_statuses(',
                'use_func' => 'tep_get_order_status_name',
            ],
        ];
    }

    protected function get_awaiting_payment_status_id() {
        return abstract_payment_module::ensure_order_status(
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_AWAITING_PAYMENT_ORDER_STATUS_ID',
            'awaiting payment (Verotel)'
        );
    }

    protected function get_payment_accepted_status_id() {
        return abstract_payment_module::ensure_order_status(
            'MODULE_PAYMENT_VEROTEL_FLEXPAY_PAYMENT_ACCEPTED_ORDER_STATUS_ID',
            'payment accepted (Verotel)'
        );
    }

    protected function redirect_to_cart() {
        tep_redirect(tep_href_link('shopping_cart.php'));
    }

    protected function get_order_status($order_id, $customer_id) {
        $order_status_query = tep_db_query(
            "SELECT orders_status FROM orders WHERE orders_id = "
                . (int)$order_id . " AND customers_id = "
                . (int)$customer_id
        );

        if (!tep_db_num_rows($order_status_query)) {
            throw new verotel_flexpay_GetOrderStatusException();
        }

        return tep_db_fetch_array($order_status_query);
    }

    protected function accept_payment($order, $order_id, $customer_id) {
        $order_status = $this->get_order_status($order_id, $customer_id);

        if ($order_status['orders_status'] != $this->cfg_order_status) {
            $GLOBALS['hooks']->register_pipeline('after');

            $sql_data = [
                'orders_id' => $order_id,
                'orders_status_id' => $this->get_payment_accepted_status_id(),
                'date_added' => 'NOW()',
                'customer_notified' => 0,
                'comments' => '',
            ];
            tep_db_perform('orders_status_history', $sql_data);

            tep_db_query(
                "UPDATE orders SET orders_status = "
                    . $this->cfg_order_status
                    . ", last_modified = NOW() WHERE orders_id = "
                    . (int)$order_id
            );

            $order->info['order_status'] = $this->cfg_order_status;
            require 'includes/system/segments/checkout/insert_history.php';

            $this->after_process();
        }

        $GLOBALS['hooks']->register_pipeline('reset');

        unset($_SESSION['cart_Verotel_FlexPay_pre_order']);
    }
}

function get_verotel_flexpay_brands() {
    return array(
        array( 'id' => 'Verotel', 'text' => 'Verotel (verotel.com)' ),
        array( 'id' => 'Bill', 'text' => 'Bill (bill.creditcard)' ),
        array( 'id' => 'BitsafePay', 'text' => 'BitsafePay (bitsafepay.com)' ),
        array( 'id' => 'CardBilling', 'text' => 'CardBilling (billing.creditcard)' ),
        array( 'id' => 'GayCharge', 'text' => 'GayCharge (gaycharge.com)' ),
    );
}

function tep_cfg_pull_down_verotel_flexpay_brand($brand_id, $key = '') {
    return tep_draw_pull_down_menu(
        (($key) ? 'configuration[' . $key . ']' : 'configuration_value'),
        get_verotel_flexpay_brands(), $brand_id
    );
}

function tep_cfg_show_verotel_flexpay_brand($value) {
    foreach (get_verotel_flexpay_brands() as $brand) {
        if ($brand['id'] == $value) {
            return $brand['text'];
        }
    }
    return $value;
}
